/** 
    * FOR DEVELOPERS WITH PARTNER ACCOUNTS:
    * If you are testing Partner-specific features, you will need your certificates installed correctly.
    * To make calls on *behalf of users*, use the impersonate_user argument on login. Do not use impersonate_user when creating users or companies.
*/

/**
    * FOR EVERYONE:
    * This code will walk you through logging in, creating a list and an email campaign, attaching that list to that email campaign, and launching it. 
    * There are several points at which you can update the content -- email addresses, email subject line, etc., but the only required edits to this code
    * are providing your username and password below. It is written to demonstrate use of these calls with as much VRAPI-specific code and as little generic code as possible.
   */

package com.verticalresponse.vrapi_java_examples;

import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.io.*;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Date;


import javax.xml.rpc.ServiceException;
import javax.xml.rpc.handler.soap.SOAPMessageContext;
import javax.xml.rpc.soap.SOAPFaultException;

import com.sun.xml.internal.xsom.impl.scd.Iterators.Map;
import com.verticalresponse.api._1_0.VRAPI.VRAPILocator;
import com.verticalresponse.api._1_0.VRAPI.VRAPIPortType;
import com.verticalresponse.api._1_0.VRAPI_xsd.*;

public class SimpleTest 
{
	// *** REPLACE THESE PARAMETERS WITH YOUR ACCOUNT CREDENTIALS ***
	static final String USERNAME = "ACCOUNT_USERNAME_HERE"; //Your account username
	static final String PASSWORD = "ACCOUNT_PASSWORD_HERE"; //Your account password
	
	// PARTNER API ONLY: Set the path to your P12 certificate you were provided in the KEYSTORE variable below,
	// and the passphrase you were provided in the KEYSTOREPASSWORD below. 
	// For more information on setting up the certificates for production installations, 
	// see http://developers.verticalresponse.com/api/soap/developers-reference/partner-resources/java-certificate-guide/
	static final String KEYSTORE = "";
	static final String KEYSTOREPASSWORD = "";
	static final String KEYSTORETYPE = "PKCS12";

	// These values refer to the one member who will be a recipient of our campaign.
	// *** TO RECEIVE THE EMAIL YOU ARE GENERATING, REPLACE THIS DATA WITH YOUR EMAIL ADDRESS *** 
	String MEMBER_EMAIL_ADDRESS = "test_email_reciepient@test.com";
	String MEMBER_FIRST_NAME = "First_Name";
	String MEMBER_LAST_NAME = "Last_Name";
	
	// If true, we stop after every step. This will allow you to examine your lists
	// and campaigns made by this program using the VR website, or a client-side program
	// such as ApiTest.java
	static final boolean pauseBetweenSteps = false;
	// This wraps the standard input stream, so we can read lines from it.
	// only used when pausing
	BufferedReader _br = null;
	
	// This variable points to the interface defining all VRAPI functions.
	VRAPIPortType _vr = null;

	// PARTNER ACCOUNT HOLDERS ONLY: Setting up the SSL certificates 
	
		// Since your PKCS12 certificate was signed by the VerticalResponse API Services certificate authority, 
		// youll need to instruct the JVM to trust this authority in order to use your PKCS12 certificate. 
		// Youll do this by creating a trust store and importing vr_api_ca.crt into it.
		// Get it here, along with helpful configuration instructions:
		// http://developers.verticalresponse.com/api/soap/developers-reference/partner-resources/java-certificate-guide/
	
	public static void setCertificate ()
	{
		try
		{
			System.setProperty ("javax.net.ssl.keyStore", KEYSTORE);
			System.setProperty ("javax.net.ssl.keyStorePassword", KEYSTOREPASSWORD);
			System.setProperty ("javax.net.ssl.keyStoreType", KEYSTORETYPE);
		}
		catch (SecurityException e)
		{
			// System.out.println ("Security violation on setting: " + key + " to: " + value);
			e.printStackTrace();
			System.exit (1);
		}
		catch (Exception e)
		{
			// System.out.println ("Exception: " + e + " on setting: " + key + " to: " + value);
			e.printStackTrace();
			System.exit (1);
		}
	} // setCertificate
	
	
	public static void main ( String[] argv ) 
	{	// Create an instance of ourselves to do all other operations
		SimpleTest simpleTest = new SimpleTest ();
		// Invoke test method with all the arguments
		simpleTest.test (argv);
	} // main
	
	public void test (String[] argv )
	{
	try{
		String sessionID = null;
		int listID = 0;
		int campaignID = 0;
		ListMember addedMember = null;
		
		// if we're doing pausing, set up some stream mummery so we can read lines from the keyboard
		// it's amazing how much work Java makes you go to to do this
		if (pauseBetweenSteps)
		{
			InputStreamReader isr = new InputStreamReader(System.in);
			_br = new BufferedReader (isr);
		}

		// obtain vrapi port, the interface which defines all VRAPI remote functions available for your application
		System.out.println ("Step 0: calling getVRAPIPort");
		VRAPIPortType vr = null;
			VRAPILocator locator = new VRAPILocator();
			vr = locator.getVRAPIPort();

		System.out.println ("getVRAPIPort successful. We now have the interface that defines the API fuctions available.\n");

		// ***************************************************************************
		// Uncomment for Partner Accounts
		// 		setCertificate ();
		// Properties are inherited from Java command line from -D
		
		// ***************************************************************************
		// LOGIN 
		// The session ID we'll get back from the login call will be used in every subsequent call. 
		System.out.println ("Step 1: calling login");

		System.out.println("Logging into account associated with the username " + USERNAME);
		
		// The session length can be set from 1 to 120 minutes.
		BigInteger sessionTimeoutMinutes = new BigInteger( "10" );
		
		// Throughout the sample code, we'll bundle up the parameters before passing them to VerticalResponse via the port. 
		LoginArgs loginArgs = new LoginArgs( USERNAME, PASSWORD, sessionTimeoutMinutes );

		sessionID = vr.login( loginArgs );
		
		System.out.println (sessionID);

		if (sessionID == null)
		{
			System.out.println ("*** login Failed");
			System.out.println ("*** System Message: " + sessionID);
			return;
		}
		System.out.println ("Login successful. Now connected with session ID = " + sessionID);
		System.out.println ();
		pause ();

		// ***************************************************************************
		// CREATE A LIST
		// We need a list to send an email to, so let's make one. List names must be unique.
		// Every unique email address added to your account will be present in the Master List.
		
		System.out.println ("Step 2: calling createList");

		String listName = "API Test List "+ System.currentTimeMillis();
		String type = "email";
		
		CreateListArgs createListArgs = new CreateListArgs( sessionID, listName, type, null, null );

		listID = vr.createList(createListArgs);
		System.out.println (listID);
		
		if (listID <= 0)
		{
			System.out.println ("*** createList failed");
			return; 
		}
		System.out.println ("Success! Your new list ID is " + listID);
		System.out.println ();
		pause ();

		// ***************************************************************************
		// ADD A LIST MEMBER
		// If you want to actually receiving your test email, change the MEMBER_EMAIL_ADDRESS variable above.
		// Most of the time you will be adding list members using the method appendFileToList, since looping over addListMember can cause significant lag.
        // Check out the docs at http://developers.verticalresponse.com for a code snippet using appendFileToList.

		System.out.println ("Step 3: calling addListMember");
		
		int member_data_needed = 3;  // How many attributes we'll be passing. No doubt your real program will not contain such a constant.
		NVPair[] member_data = new NVPair [member_data_needed];  // Allocate array of needed size
		int ix = 0;
		// Create Name/Value pairs for each piece of data.
		member_data [ix++] = new NVPair ("email_address", MEMBER_EMAIL_ADDRESS);
		member_data [ix++] = new NVPair ("first_name", MEMBER_FIRST_NAME);
		member_data [ix++] = new NVPair ("last_name", MEMBER_LAST_NAME);

		// Now we can allocate the ListMember
		ListMember list_member = new ListMember ( listID, listName, type, member_data );
			list_member.setList_id(listID);
			list_member.setMember_data(member_data);
			
		AddListMemberArgs addListMemberArgs = new AddListMemberArgs ();
			addListMemberArgs.setList_member(list_member);
			addListMemberArgs.setSession_id(sessionID);

	    addedMember = vr.addListMember( addListMemberArgs );

		if (addedMember == null)
		{
			System.out.println ("*** Unfortunately, addListMember failed");
			return;
		}

		// Let's get back the member data for the contact we added and take a look at it.
		NVPair[] addedData = addedMember.getMember_data();
		if (addedData == null)
		{
			System.out.println ("*** Strange, the returned member data is null");
			return;
		}
		
		System.out.println ("Here's the list member you added: ");
		Hashtable addedHt = printMemberData (addedData);
		
		pause ();

		// ***************************************************************************
		// CREATE AN EMAIL
		// You can replace any of the content with your own. 
		System.out.println ("Step 4: calling createEmailCampaign");

		Email email = new Email ();
			email.setName ("Java Test " + System.currentTimeMillis());
			email.setFrom_label ("VerticalResponse Java Sample Code");
			email.setSubject("Java API Test Email");
			email.setFreeform_html("<h2>This is the VerticalResponse API sample Java code!</h2>");
			email.setFreeform_text("This is the VerticalResponse API sample Java code!");
			email.setEmail_type("freeform");
			email.setReply_to_email(USERNAME);
			email.setPostal_addr_1("50 Beale Street");
			email.setPostal_city("San Francisco");
			email.setPostal_code("94105");
			email.setPostal_state("CA");
			email.setPostal_country("USA");
			email.setTested(true);
			email.setPreviewed_html(true);
			email.setPreviewed_text(true);

		CreateEmailArgs createEmailArgs = new CreateEmailArgs( sessionID, email);
		campaignID = vr.createEmail( createEmailArgs );

		if (campaignID < 0)
		{
			System.out.println ("*** createEmail FAILED");
			return;
		}
		System.out.println ("Email created! Returned campaign ID " + campaignID);
		System.out.println ();
		
		pause ();

		// ***************************************************************************
	    // ATTACH A LIST TO CAMPAIGN
		// This attaches the list you made above to the campaign you just created. You can send an email to more than one list.

		System.out.println ("Step 5: calling setCampaignLists");

		int[] listIDs = new int[1];  
		listIDs [0] = listID; 

		SetCampaignListsArgs setCampaignListsArgs = new SetCampaignListsArgs();
			setCampaignListsArgs.setSession_id(sessionID);
			setCampaignListsArgs.setCampaign_id(campaignID);
			setCampaignListsArgs.setList_ids(listIDs);
		
		boolean success;
		
		success = vr.setCampaignLists( setCampaignListsArgs );

		if (!success){
			System.out.println ("setCampaignLists failed. Hmm.");
			return;
		}
		System.out.println ("Success! The list is set.");
		System.out.println ();
		
		pause ();
		
		// ***************************************************************************
		// LAUNCH CAMPAIGN
		// This will send the campaign to the VerticalResponse campaign checking team.
        // They make sure that all campaigns that we send out comply with US federal law and our terms of service.
        // If the campaign is declined, you'll receive an email at your log in email address explaining why.
		

		System.out.println ("Step 6: calling launchEmailCampaign");
		
		LaunchEmailCampaignArgs launchEmailCampaignArgs = new LaunchEmailCampaignArgs();
			launchEmailCampaignArgs.setSession_id(sessionID);
			launchEmailCampaignArgs.setCampaign_id(campaignID);

		LaunchEmailCampaignResult result;
		result = vr.launchEmailCampaign( launchEmailCampaignArgs );

		if (result == null)
		{
			System.out.println ("Unfortunately, your email was not launched.");
			return;
		}
		System.out.println ("Email campaign successfully launched.");
		System.out.println ("This will send the campaign to the VerticalResponse campaign checking team.");
        System.out.println ("They make sure that all campaigns that we send out comply with US federal law and our terms of service.");
		System.out.println ("If the campaign is declined, you'll receive an email at your log in email address explaining why.");
		System.out.println ();
        
        System.out.println ("Let's take a look at some stastics about your campaign:");
		printLaunchEmailCampaignResult (result);

		System.out.println ();
		System.out.println ("At this point, the person on your list should receive the email, and the account email address will get an emailed campaign report from us.");
		System.out.println ("You should wait until you receive these emails before proceeding.");
		System.out.println ("If you proceed before receiving the campaign report, the server will likely raise a fault");
	}
	catch ( RemoteException re ) {
		re.printStackTrace();
	}
	catch ( Exception e ) {
		return;
	}
		} // test
	
			

	// HERE BEGIN THE HELPER METHODS. They are provided for your convenience but are not part of the API. 
			
	/**
	 * Reads from a given input stream the given number of bytes,
	 * converting them into a string, and appending that string
	 * onto given StringBuffer.
	 * @param sb StringBuffer to which to append content
	 * @param is InputStream from which to read content
	 * @param len int for expected length of content
	 * @return int indicating total length read, -1 if error (other than EOF) occurred
	 */
	int appendStream (StringBuffer sb, InputStream is, int len)
	{
		int maxlen = 10000; // temp for now
		byte[] buf = new byte [maxlen];
		int soFar = 0;  // none read at all

		// read bytes from this stream until requested length satisfied
		// or error or EOF
		while (soFar < len)
		{
			try
			{
				int numRead = is.read(buf, 0, maxlen);
				if (numRead == 0)
				{
					System.out.println("numRead is zero");
					return (soFar);
				}
			
//				System.out.println("numRead = " + numRead);
				
				// Append the contents of the buffer onto the string buffer
				String str = new String (buf, 0, numRead);
				sb.append (str);
				soFar += numRead;
			}
			catch (EOFException e)
			{
				return (soFar);
			}
			catch (Exception e)
			{
				System.out.println("Exception in appendStream = " + e);
				return (-1);
			}
		}
		
		return (soFar);
	} // appendStream
	

	/*
	 * Loads contents pointed to by a FileSpec
	 * @param fs FileSpec for downloaded file
	 * @param sb StringBuffer to which to append contents. We do not empty it out first, that's your problem.
	 * @return int for # of bytes downloaded, -1 if error
	 */
	int loadContents (FileSpec fs, StringBuffer sb)
	{
		String urlStr = fs.getLocation();
//		System.out.println ("loadContents: filename = " + fs.getFilename() + ", location = " + urlStr);
		
		// Now try to load the file pointed to by the location value
		HttpURLConnection urlconn = null;
		try
		{
			URL url = new URL(urlStr);
			urlconn = (HttpURLConnection)url.openConnection();
			// Now load the file
			int contentLength = urlconn.getContentLength();
			if (contentLength == 0)
			{
				System.out.println("Content length is zero");
				return (0);
			}
			System.out.println("*** Content length = " + contentLength);
			
			String contentType = urlconn.getContentType();
			if (contentType == null)
			{
				System.out.println("*** Content type is null");
				return (0);
			}
//			System.out.println("Content type = " + contentType);
			
			// Extract actual content object from the url connection. We don't yet know its type
			Object contentObj = urlconn.getContent();
			if (contentObj == null)
			{
				System.out.println("*** Content object is null");
				return (0);
			}
	
			if (contentObj instanceof InputStream)
			{
				// It's an input stream, read its contents and append to our budding string buffer
				int totalRead = appendStream (sb, (InputStream) contentObj, contentLength);
				return (totalRead);  // only really good exit
			}
			else
			{
				// We don't know how to read any other type of object.
				System.out.println("*** Content object is of unknown type");
				return (-1);
			}
		}
		catch(MalformedURLException mfe)
		{
			System.out.println("*** MalformedURL Exception, url = " + urlStr);
			return (-1);
		}				
		catch(IOException ioe)
		{
			System.out.println("*** IO Exception, url = " + urlStr);
			ioe.printStackTrace();
			return (-1);
		}
		
	} // loadContents
	

	/**
	 * Appends to buffer _sb a printable form of an array of Java strings
	 * @param name String for name of array
	 * @param array Array of stuff to print (doesn't have to be strings of course), but must implement toString() reasonably
	 */
	void printArray (Object[] array )
	{
		if (array == null)
		{
			System.out.println ("(NULL)");
			return;
		}
		
		if (array.length == 0)
		{
			System.out.println ("(EMPTY)");
			return;
		}
		
		for (int j = 0; j < array.length; j++)
		{
			System.out.print("[");
			System.out.print(j);
			System.out.print("]: ");
			System.out.println(array [j].toString());  // this is the weak link
		}
		
	} // printArray


	/**
	 * Prints a member_data type array with one name-value pair per line.
	 * As a useful side-effect, loads a Hashtable with each pair found.
	 * Returns the filled hashtable.
	 * @param pairs NVPair[] array of pair data
	 * @return Hashtable containing loaded pair data.
	 */
	Hashtable printMemberData (NVPair[] pairs)
	{
		Hashtable ht = new Hashtable (pairs.length);  // Alloc hashtable, in this case we know length.
		for (int j = 0; j < pairs.length; j++)
		{
			NVPair pair = pairs [j];
			System.out.print ("pair [" + j + "]: ");
			
			if (pair == null)
			{
				System.out.println ("(NULL)");
				continue;
			}
	
			// Extract values from the pair.
			String name = pair.getName();
			String value = pair.getValue();
			
			if ((name == null) || (value == null))
			{
				System.out.println ("(NULL)");
				continue;
			}
	
			System.out.println ("name=" + name + ", value=" + value);

			// Put into hashtable ht the name/value pair as key/value pair.
			ht.put (name, value);
		}
		
		System.out.println ();
		return (ht);
	} // printMemberData


    /**
	 * Prints an EmailCampaignResult
	 * @param name String for name of LaunchEmailCampaignResult
	 * @param result EmailCampaignResult to be printed.
	 */
	void printLaunchEmailCampaignResult (LaunchEmailCampaignResult result)
	{
		System.out.println ("total_size=" + result.getTotal_size ());
		System.out.println ("total_mailable=" + result.getTotal_mailable ());
		System.out.print ("merge_fields = ");
		String[] fields = result.getMerge_fields ();
		if (fields == null)
		{
			System.out.println ("(NULL)");
			return;
		}
		
		if (fields.length == 0)
		{
			System.out.println ("(EMPTY)");
			return;
		}

		System.out.println ("array of length: " + fields.length);
		for (int i = 0; i < fields.length; i++)
		{
			System.out.println ("merge_fields[" + i + "] = " + fields[i]);
		}
		System.out.println ();			
	} // printLaunchEmailCampaignResult


	/**
	 * Appends to buffer _sb the contents of a FileSpec
	 * @param name String for name of FileSpec
	 * @param fs FileSpec to print
	 */
	void printFileSpec (FileSpec fs)
	{
		System.out.println ("Filename=" + fs.getFilename());
		System.out.println ("Delimiter=" + fs.getDelimiter());
		System.out.println ("Location=" + fs.getLocation());
		System.out.println ("Contents=" + fs.getContents ());
		// Download the contents
		StringBuffer sb = new StringBuffer(); 
		int totalDownloaded = loadContents (fs, sb);  // Load into sb contents pointed to by this fs
		if (totalDownloaded < 0)
			System.out.println ("*** Could not download contents");
		else
		{
			// run thru all the lines
			int ixnext = 0;
			int lineno = 0;
			for (int ix = 0; ix < sb.length(); ix = ixnext)
			{
				// find a nl
				ixnext = sb.indexOf("\n", ix);
				if (ixnext < 0)
					ixnext = sb.length();
				String line = sb.substring(ix, ixnext);
/*				
				pause();
				System.out.println ("******** line=" + lineno);
				for (int j = 0; j < line.length(); j++)
				{
					char ch = line.charAt(j);
					System.out.println ((int) ch);
				}
*/				
				System.out.println (line);
				// skip over nl
				ixnext++;
				lineno++;
			}
//			System.out.println ("******** # of lines=" + lineno);
		}
	} // printFileSpec
	
	
    /**
	 * Prints a DownloadCampaignRecipientResultsResult
	 * @param name String for name of DownloadCampaignRecipientResultsResult
	 * @param result DownloadCampaignRecipientResultsResult to be printed.
	 */
    // void printDownloadCampaignRecipientResultsResult (DownloadCampaignRecipientResultsResult result)
    //   {
    //   	System.out.println ("campaign_id=" + result.getCampaign_id ());
    //   	System.out.println ("num_records=" + result.getNum_records ());
    // 		printFileSpec (result.getContents());
    // 		System.out.println ("included_fields=");
    // 		printArray (result.getIncluded_fields ());
    // 		System.out.println ("restricted_responses=");
    // 		printArray (result.getRestricted_responses ());
    // 		System.out.println ("restricted_lists=");
    // 		int[] restricted_lists = result.getRestricted_lists();
    // 		if (restricted_lists == null)
    // 			System.out.println ("(NULL)");
    // 		else
    // 		{
    // 			for (int i = 0; i < restricted_lists.length; i++)
    // 			{
    // 				System.out.println ("restricted_lists[" + i + "] = " + restricted_lists[i]);
    // 			}
    // 		}
    //   } // printDownloadCampaignRecipientResultsResult
    

	/*
	 * Pauses if the pauseBetweenSteps is true. Does nothing if false.
	 * Any user input of one character will be swallowed, then we return.
	 */
	void pause ()
	{
		if (!pauseBetweenSteps)
			return;
		System.out.print ("Press <enter> to continue:");
		try
		{
			_br.readLine();
		}
		catch (IOException ioe)
		{
			System.out.println ("pause - could not read user input");
			ioe.printStackTrace();
		}
	} // pause

} // class SimpleTest
